package org.qtunes.daap;

import org.qtunes.db.*;
import java.util.*;

/**
 * A Group is a set of Tracks, basically the equivalent to a previously run search,
 * usually an Album.
 */
class Group {

    private final String album, artist;
    private final long persistent;
    private BitSet tracks;
    private int index;

    private Group(long persistent, String album, String artist) {
        this.album = album;
        this.artist = artist;
        this.persistent = persistent;
        this.tracks = new BitSet();
    }

    void addTrack(int track) {
        tracks.set(track);
    }

    void setIndex(int i) {
        this.index = i;
    }

    int getIndex() {
        return index;
    }

    String getArtist() {
        return artist;
    }

    String getAlbum() {
        return album;
    }

    long getPersistentId() {
        return persistent;
    }

    BitSet getTracks() {
        return tracks;
    }

    /**
     * Create 1 or more Groups which contain the tracks from the specifiec playlist
     */
    static List<Group> createGroupList(DaapServer server, Playlist playlist) {
        Database db = server.getDatabase();
        Map<Long,Group> m = new LinkedHashMap<Long,Group>();
        ArrayList<Group> list = new ArrayList<Group>();

        List<Track> tracks = playlist.getTracks();
        for (int i=0;i<tracks.size();i++) {
            Track track = tracks.get(i);
            String album = track.get(Field.Album);
            String artist = (String)track.get(DaapField.GroupArtist);
            long groupid = DaapField.digest64(album, artist);
            Group group = m.get(groupid);
            if (group==null) {
                group = new Group(groupid, album, artist);
                m.put(groupid, group);
                list.add(group);
                server.addGroup(group);
            }
            group.addTrack(track.getIndex());
        }
        return list;
    }

}
