package org.qtunes.daap;

import java.util.*;
import java.io.*;

/**
 * A type of Block that represents a List of other Blocks
 */
class ListBlock extends Block {

    private List<Block> values = new ArrayList<Block>();

    public String toString() {
        return getName()+"="+values+"("+length()+")";
    }

    void set(int ix, Block block) {
        values.set(ix, block);
    }

    Block get(int ix) {
        return values.get(ix);
    }

    ListBlock get(Blocks.BlockDefL def) {
        Block block;
        for (int i=0;i<values.size();i++) {
            if ((block=get(i)).getCode()==def.getCode()) {
                return (ListBlock)block;
            }
        }
        return null;
    }

    NumericBlock get(Blocks.BlockDefN def) {
        Block block;
        for (int i=0;i<values.size();i++) {
            if ((block=get(i)).getCode()==def.getCode()) {
                return (NumericBlock)block;
            }
        }
        return null;
    }

    StringBlock get(Blocks.BlockDefS def) {
        Block block;
        for (int i=0;i<values.size();i++) {
            if ((block=get(i)).getCode()==def.getCode()) {
                return (StringBlock)block;
            }
        }
        return null;
    }

    void add(Block block) {
        values.add(block);
    }

    int size() {
        return values.size();
    }

    int length() {
        int len = 0;
        for (int i=0;i<size();i++) {
            len += get(i).length() + 8;
        }
        return len;
    }

    void read(InputStream in, int length) throws IOException {
        while (length > 0) {
            Block block = Blocks.read(in, this);
            add(block);
            length -= block.length() + 8;
        }
    }

    void writeContent(OutputStream out) throws IOException {
        for (int i=0;i<size();i++) {
            get(i).write(out);
        }
    }

    void dump(Appendable sb, String prefix) throws IOException {
        StringBuffer space = null;
        sb.append(getName());
        sb.append("=[");
        String prefix2 = prefix==null ? null : prefix + " ";
        for (int i=0;i<size();i++) {
            if (prefix2!=null) {
                sb.append(prefix2);
            } else if (i > 0) {
                sb.append(", ");
            }
            get(i).dump(sb, prefix2);
        }
        if (prefix!=null) {
            sb.append(prefix);
        }
        if (size() > 0) {
            sb.append("]("+length()+")");
        } else {
            sb.append("]");
        }
    }

}
