package org.qtunes.daap;

import org.qtunes.player.*;

class PlayerState {

    private final DaapServer server;
    private final Player player;
    private final Revision revision;
    private DaapPlaylist playlist;

    PlayerState(Player player, DaapServer server) {
        this.player = player;
        this.server = server;
        this.revision = new Revision(server, 2);            // Arbitrary >1
    }

    synchronized DaapPlaylist getPlaylist() {
        if (playlist == null) {
            playlist = DaapPlaylist.getNowPlayingPlaylist(server, player);
        }
        return playlist;
    }

    Revision getRevision() {
        return revision;
    }

    void bumpPlayer() {
        revision.bump();
    }

    void bumpDatabase() {
        revision.bump();
    }

    void start() {
        player.getContext().addListener(revision);
        server.getServiceRegistrar().addPlayer(player);
    }

    void stop() {
        player.getContext().removeListener(revision);
        server.getServiceRegistrar().removePlayer(player);
    }

}
