package org.qtunes.daap;

import org.qtunes.core.*;
import java.util.Map;

/**
 * This class handles update and playstatusupdate. These requests
 * come in specifying a revision number. If we are currently at
 * that revision we hold until it changes (if not, we return immediately).
 * When we return we include the current revision number.
 *
 * Exact logic behind this is unclear
 */
class Revision implements Listener {
 
     private final DaapServer daap;
     private volatile int revision;

     Revision(DaapServer daap, int initial) {
         this.daap = daap;
         this.revision = initial;
     }

     /**
      * Increase the revision and notify anyone waiting on it
      */
     synchronized void bump() {
         revision++;
         notifyAll();
     }

     int getRevision() {
         return Session.getCurrentSession()==Session.NONE ? 0 : revision;
     }

     /**
      * Cause this method to pause until this Revision
      * is greater than the specified "waiting" revision
      */
     void waitUntilGreater(int waiting) {
         while (waiting >= revision && Session.getCurrentSession()!=Session.NONE) {
             synchronized(this) {
                 try {
                     wait();
                 } catch (InterruptedException e) { }
             }
         }
     }

     public synchronized void handleEvent(String topic, Map properties) {
         if (topic.equals("expire") && properties.get("service")==daap.getAuth()) {
             notifyAll();
         }
     }

     public synchronized void cancel() {
         revision = Integer.MAX_VALUE;
         notifyAll();
     }

}
