package org.qtunes.db;

import org.qtunes.core.*;
import java.util.*;

/**
 * A Matcher determines whether the fields in a {@link FieldMap} match an 
 * already existing track. This can be done via a number of ways - the most
 * obvious is on FileName, but if a file is
 * renamed this won't register as a rename but an add and delete. A better
 * approach would be to store a checksum of the audio data in the file, 
 * as this doesn't change when the file is tagged or renamed.
 */
public interface Matcher {

    /**
     * Get the key that can be used to uniquely identify a track with the specified fields
     */
    public Object getKey(FieldMap map);

    /**
     * Get the key that can be used to uniquely identify the specified track
     */
    public Object getKey(Track track);
}
