package org.qtunes.db;

import org.qtunes.core.*;
import java.util.*;
import java.io.Serializable;

/**
 * A Playlist is a list of tracks, which may be manually or magically managed.
 * The basic interface allows the tracks indices to be read, but doesn't
 * care how they are determined. As well as the obvious Playlists, each {@link org.qtunes.player.Player}
 * will probably have a Playlist listing the tracks that are queued.
 */
public interface Playlist extends Serializable {

    /**
     * Return the name of the Playlist
     */
    public String getName();

    /**
     * Set the name of the Playlist, as returned by {@link #getName} (optional operation)
     */
    public void setName(String name);

    /**
     * Get the revision number of the Playlist. Every modification to the list should increase this
     */
    public int getRevision();

    /**
     * Get a unique ID for this playlist which should never change.
     */
    public int getUniqueID();

    /**
     * Get the list of tracks in this Playlist
     */
    @ControlMethod (
        params = {},
        help = "Return the List of tracks in this Playlist",
        auth = "*"
    )
    public List<Track> getTracks();

    /**
     * Get a new Playlist based on the current one, but sorted and filtered as specified.
     * This playlist doesn't need to live-update, and this method should be cheap, ie. it
     * should not require evaluating the whole list of tracks returned by getList to create.
     * @param sort the field to sort by, eg "Title", "Artist", or null for no sorting
     * @param filter the text to filter on, or null for no filtering
     */
    @ControlMethod (
        params = {"sort", "filter"},
        help = "Return the List of tracks in this Playlist - 'sort' is the key to sort by, eg 'Title','Artist', and 'filter' is the text to filter on",
        auth = "*"
    )
    public Playlist sort(String sort, String filter);

    /**
     * Get the number of tracks in this Paylist
     */
    @ControlMethod (
        params = {},
        help = "Get the number of Tracks in this Playlist"
    )
    public int size();


}
