package org.qtunes.db;

import org.qtunes.core.util.AbstractMatcher;

/**
 * A {@link AbstractMatcher} which matches against the {@link Field fields} in a {@link Track} by name.
 * This can be used for {@link SmartPlaylist} or similar. Some examples:
 * <pre class="example">
 * Artist = "Prince"
 * (Genre = "Dance" or Genre = "House") and Year < 2000
 * </pre>
 */
public class TrackMatcher extends AbstractMatcher<Track> {

    /**
     * Create a new TrackMatcher with the specified filter
     * @param filter the filter
     */
    public TrackMatcher(String filter) {
        super(filter);
    }

    @SuppressWarnings("unchecked")
    public Object getProperty(Track track, String property) {
        if (property.equals("Age")) {
            Integer touch = track.get(Field.LastModified);
            if (touch != null) {
                long touchtime = touch.intValue() * 1000L;
                return Integer.valueOf((int)((System.currentTimeMillis() - touchtime) / 86400000L));
            } else {
                return null;
            }
        } else {
            AbstractField field = Field.getByName(property);
            if (field == null) {
                if (field instanceof Field && ((Field)field).getType() == Boolean.class) {
                    return false;
                } else {
                    return null;
                }
            } else {
                return track.get(field);
            }
        }
    }

}
