package org.qtunes.ff.spi.mp3;

import java.io.*;
import java.nio.*;
import java.util.*;
import java.text.*;

class TagTimestamp extends TagText {

    private static final String[] formats = new String[] { "yyyy-MM-dd'T'HH:mm:ss", 
          "yyyy-MM-dd'T'HH:mm", "yyyy-MM-dd'T'HH", "yyyy-MM-dd", "yyyy-MM", "yyyy" };

    private long time;
    private boolean read, invalid;

    TagTimestamp() {
    }

    long getTimestamp() throws IOException {
        if (!read) {
            String value = getValue();
            try {
                Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
                for (int i=0;i<formats.length;i++) {
                    SimpleDateFormat format = new SimpleDateFormat(formats[i]);
                    format.setCalendar(cal);
                    Date date = format.parse(value, new ParsePosition(0));
                    if (date!=null) {
                        time = date.getTime();
                        read = true;
                        break;
                    }
                }
            } catch (RuntimeException e) {
                invalid = true;
                throw new IOException("Unable to parse time \""+value+"\"", e);
            }
            if (!read) {
                throw new IOException("Unable to parse time \""+value+"\"");
            }
        }
        return time;
    }

    boolean isInvalid() {
        return invalid;
    }

    Calendar getCalendar() throws IOException {
        long time = getTimestamp();
        Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
        cal.setTime(new Date(time));
        return cal;
    }

    public String toString() {
        try {
            long time = getTimestamp();
            DateFormat df = DateFormat.getDateTimeInstance();
            df.setCalendar(Calendar.getInstance(TimeZone.getTimeZone("UTC")));
            return "["+getName()+" time="+df.format(new Date(time))+"]";
        } catch (IOException e) {
            return super.toString();
        }
    }

}
