package org.qtunes.speaker.spi.dummy;

import org.qtunes.core.*;
import org.qtunes.speaker.*;
import org.qtunes.player.Player;
import javax.sound.sampled.*;
import java.io.*;
import java.io.*;
import java.util.*;

public class SpeakerImpl implements Speaker {

    private ServiceContext context;
    private Player player;
    private volatile Exception error; 
    private int volumeadjust, bytespersecond;
    private long opentime, numbytes;

    private void setError(Exception e) {
        this.error = e;
        if (error != null) {
            context.warn("Speaker Error", e);
            context.fireEvent("speakerError", new Object[] { "exception", this.error });
        }
    }

    public Exception getError() {
        return error;
    }

    public Player getPlayer() {
        return player;
    }

    public void setPlayer(Player player) {
        if (this.player != null) {
            this.player.removeSpeaker(this);
        }
        this.player = player;
        if (player != null) {
            player.addSpeaker(this);
        }
    }

    public String getDisplayName() {
        return context.getProperty("display");
    }

    public int getUniqueId() {
        try {
            return Integer.parseInt(context.getProperty("uid"));
        } catch (Exception e) {
            return getDisplayName().hashCode() & 0x7FFFFFFF;
        }
    }

    public boolean hasGain() {
        return true;
    }

    public int getDelay() {
        return 0;
    }

    public int getBufferSize() {
        return 1408;
    }

    public void write(byte[] buf, int off, int len) {
        if (opentime == 0) {
            setError(new IllegalStateException("Write without open"));
        } else {
            numbytes += len;
            long targettime = opentime + (numbytes * 1000 / bytespersecond);
            long diff;
            while ((diff = targettime - System.currentTimeMillis()) > 0) {
                try {
                    Thread.sleep(diff);
                } catch (Exception e) { }
            }
        }
    }

    public void flush() {
        context.fireEvent("speakerStopped", null);
    }

    public void drain() {
        context.fireEvent("speakerStopped", null);
    }

    public boolean isOpen() {
        return opentime != 0;
    }

    public void close() {
        opentime = 0;
        bytespersecond = 0;
    }


    public void open(float sampleRate, int sampleSizeInBits, int channels, boolean signed, boolean bigEndian) {
        bytespersecond = (int)(sampleSizeInBits * channels * sampleRate / 8);
        opentime = System.currentTimeMillis();
    }

    public void setVolumeAdjustment(int volumeadjust) {
        this.volumeadjust = volumeadjust;
        context.putProperty("volumeadjust", Integer.toString(volumeadjust));
    }

    public int getVolumeAdjustment() {
        return volumeadjust;
    }

    public void setGain(float gain) {
    }

    //----------------------------------------------------------------------------

    public void startService(ServiceContext context) {
        this.context = context;
        if (getDisplayName() == null) {
            throw new IllegalStateException("No \"display\" property");
        }
        try {
            volumeadjust = Integer.parseInt(context.getProperty("volumeadjust"));
        } catch (Exception e) { }
    }

    public void stopService(ServiceContext context) {
    }

    public ServiceContext getContext() {
        return context;
    }

    public Map<String,Object> reportState() {
        Map<String,Object> map = new LinkedHashMap<String,Object>();
        map.put("name", getDisplayName());
        map.put("volumeadjust", getVolumeAdjustment());
        return map;
    }

}
