package org.qtunes.webplayer;

import java.net.*;
import java.util.*;
import java.io.*;
import org.qtunes.auth.*;
import org.qtunes.core.*;
import java.security.*;
import org.qtunes.core.util.Base64;

class WebAuthorizer implements Authorizer {

    private final Map<String,WebAuth> authorizedusers;
    private final WebPlayer player;

    WebAuthorizer(WebPlayer player) {
        this.player = player;
        this.authorizedusers = Collections.synchronizedMap(new HashMap<String,WebAuth>());
    }

    public Collection<String> getAllowances(Session session) {
        if (session.getService()==player) {
            Map properties = session.getProperties();
            String user = (String)properties.get("user");
            String password = (String)properties.get("password");
            for (Iterator<Map.Entry<String,WebAuth>> i = authorizedusers.entrySet().iterator();i.hasNext();) {
                Map.Entry<String,WebAuth> e = i.next();
                String authuser = e.getKey();
                if (authuser.equals(user)) {
                    String authuserpass = e.getValue().userpass;
                    String[] allowances = e.getValue().allowances;
                    try {
                        MessageDigest md5 = MessageDigest.getInstance("MD5");
                        md5.update(user.getBytes("UTF-8"));
                        md5.update(password.getBytes("UTF-8"));
                        byte[] clientdigest = md5.digest();
                        if (Arrays.equals(Base64.decode(authuserpass), clientdigest)) {
                            return Arrays.asList(allowances);
                        }
                    } catch (IOException e2) {
                        throw new RuntimeException(e2);
                    } catch (NoSuchAlgorithmException e2) {
                        throw new RuntimeException(e2);
                    }
                }
            }
        }
        return null;
    }

    private static final class WebAuth implements Serializable {
        final String userpass;
        final String[] allowances;
        WebAuth(String userpass, String[] allowances) {
            this.userpass = userpass;
            this.allowances = allowances;
        }
    }

    /**
     * Add an authorized user and password to this Service. The supplied parameters
     * are the username and the cleartext password.
     * @param user the username
     * @param password the password
     * @param allowances the list of {@link Authorizer#getAllowances Allowances} this user has
     */
    public void addUser(String user, String password, String[] allowances) {
        String userpass;
        try {
            MessageDigest digest = MessageDigest.getInstance("MD5");
            digest.update(user.getBytes("UTF-8"));
            digest.update(password.getBytes("UTF-8"));
            userpass = Base64.encode(digest.digest(), true);
            authorizedusers.put(user, new WebAuth(userpass, allowances));
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException(e);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    public void removeUser(String user) {
        authorizedusers.remove(user);
    }

}
