Ext.ns("QTunes");
QTunes.PlaylistPanel = Ext.extend(Ext.tree.TreePanel, (function() {

var qtunes;
var playlist;

var makePopup = function(me, qtunes, playlist) {
    var popup = new Ext.menu.Menu();

    if (!playlist.isNowPlaying()) {
        popup.addItem(new Ext.menu.Item({
            text: "Play",
            handler: function(n, e) {
                me.playPlaylist(playlist);
            }
        }));
        popup.addItem(new Ext.menu.Item({
            text: "Enqueue",
            handler: function(n, e) {
                me.enqueuePlaylist(playlist);
            }
        }));
    }
    if (!playlist.isRoot() && !playlist.isNowPlaying()) {
        popup.addItem(new Ext.menu.Item({
            text: "Delete",
            handler: function(n, e) {
                qtunes.actionDeletePlaylist(playlist);
            }
        }));
        popup.addItem(new Ext.menu.Item({
            text: "Rename",
            handler: function(n, e) {
                me.renamePlaylist(playlist);
            }
        }));
    }
    if (playlist.isSmart() && !playlist.isRoot() && !playlist.isNowPlaying()) {
        popup.addItem(new Ext.menu.Item({
            text: "Edit Query",
            handler: function(n, e) {
                me.editSmartPlaylist(playlist);
            }
        }));
    }

    if (true || playlist.isRoot()) {
        if (popup.items && popup.items.length > 0) {
            popup.addItem(new Ext.menu.Separator({ }));
        }
        popup.addItem(new Ext.menu.Item({
            text: "New Smart Playlist",
            handler: function(n, e) {
                me.editSmartPlaylist();
            }
        }));
    }

    return popup;
};

return {
    constructor: function(config) {
        qtunes = config.qtunes;
        var that = this;

        config = Ext.apply({
            rootVisible: false,
            lines: false,
            collapsible: false,
            bodyBorder: false,
            border: false,
            loader: new Ext.tree.TreeLoader(),
            root: new Ext.tree.AsyncTreeNode({
                expanded: true,
                children: [ ]
            }),
            listeners: {
                click: function(item) {
                    qtunes.setPlaylist(item.playlist);
                },
                contextmenu: function(item, e) {
                    e.stopEvent();
                    qtunes.setPlaylist(item.playlist);
                    var popup = makePopup(that, qtunes, item.playlist);
                    popup.showAt(e.getXY());
                },
                dblclick: function(item, e) {
                    if (!item.playlist.isNowPlaying()) {
                        var currentplaylist = qtunes.getTrackPanel().getPlaylist();
                        var sortfield, sortdir, filter;
                        if (currentplaylist == playlist) {
                            var sortinfo = qtunes.getTrackPanel().getSort();
                            sortfield = sortinfo.field;
                            sortdir = sortinfo.direction;
                        }
                        qtunes.actionPlayPlaylist(qtunes.getPlayer(), item.playlist, sortfield, sortdir, filter);
                        qtunes.setPlaylist(qtunes.getNowPlaylist());
                    }
                }
            }
        }, config);
        QTunes.PlaylistPanel.superclass.constructor.call(this, config);
    },

    setPlaylist: function(newplaylist) {
        if (newplaylist != playlist) {
            qtunes.debug("PlaylistPanel.setPlaylist("+newplaylist+")");
            for (var i=0;i<this.getRootNode().childNodes.length;i++) {
                var node = this.getRootNode().childNodes[i];
                if (node.playlist == newplaylist) {
                    playlist = newplaylist;
                    if (!node.isSelected()) {
                        node.select();
                    }
                }
            }
        }
    },

    addPlaylist: function(newplaylist) {
        qtunes.debug("PlaylistPanel.addPlaylist("+newplaylist+")");
        var node = new Ext.tree.TreeNode({
            text: newplaylist.getDisplay(),
            leaf: true,
            iconCls: newplaylist.isNowPlaying() ? "playlist-nowplaying" : newplaylist.isRoot() ? "playlist-root" : newplaylist.isSmart() ? "playlist-smart" : "playlist-manual"
        });
        var isfirst = playlist == null;
        node.playlist = newplaylist;
        this.getRootNode().appendChild(node);
	this.getRootNode().sort(function(n1, n2) {
	    if (n1.playlist == qtunes.getRootPlaylist()) {
	        return -1;
	    } else if (n2.playlist == qtunes.getRootPlaylist()) {
	        return 1;
	    } else if (n1.playlist == qtunes.getNowPlaylist()) {
	        return -1;
	    } else if (n2.playlist == qtunes.getNowPlaylist()) {
	        return 1;
	    } else {
		return n1.playlist.getDisplay().toLowerCase().localeCompare(n2.playlist.getDisplay().toLowerCase());
	    }
	});
        if (isfirst) {
            qtunes.setPlaylist(newplaylist);
        }
    },

    removePlaylist: function(newplaylist) {
        qtunes.debug("PlaylistPanel.removePlaylist("+newplaylist+")");
        for (var i=0;i<this.getRootNode().childNodes.length;i++) {
            var node = this.getRootNode().childNodes[i];
            if (node.playlist == newplaylist) {
                if (node.isSelected()) {
                    qtunes.setPlaylist(this.getRootNode().firstChild.playlist);
                }
                this.getRootNode().removeChild(node, true);
            }
        }
    },

    playPlaylist: function(playlist) {
        var currentplaylist = qtunes.getTrackPanel().getPlaylist();
        var sortfield, sortdir, filter;
        if (currentplaylist == playlist) {
            var sortinfo = qtunes.getTrackPanel().getSort();
            sortfield = sortinfo.field;
            sortdir = sortinfo.direction;
        }
        qtunes.actionPlayPlaylist(qtunes.getPlayer(), playlist, sortfield, sortdir, filter);
    },

    enqueuePlaylist: function(playlist) {
        var currentplaylist = qtunes.getTrackPanel().getPlaylist();
        var sortfield, sortdir, filter;
        if (currentplaylist == playlist) {
            var sortinfo = qtunes.getTrackPanel().getSort();
            sortfield = sortinfo.field;
            sortdir = sortinfo.direction;
        }
        qtunes.actionEnqueuePlaylist(qtunes.getPlayer(), playlist, sortfield, sortdir, filter);
    },

    renamePlaylist: function(playlist) {
        Ext.Msg.prompt("Name", "Playlist Name:", function(btn, text) {
            if (btn == 'ok' && text.length > 0) {
                if (qtunes.getNamedPlaylist(text) != playlist) {
                    Ext.Msg.alert("Error", "Playlist \""+text+"\" already exists");
                } else {
                    qtunes.actionRenamePlaylist(playlist, text);
                }
            }
        });
    },

    editSmartPlaylist: function(playlist) {
        Ext.Msg.prompt("Name", "Query:", function(btn, text) {
            if (btn == 'ok' && text.length > 0) {
                qtunes.actionCreateSmartPlaylist(playlist.getName(), text);
            }
        });
    },

    editSmartPlaylist: function(playlist) {
        var window = new Ext.Window({
            title: "New Smart Playlist",
            layout: 'fit',
            height: 140,
            width: 500,
            closable: true,
            resizable: true,
            draggable: true,
            items: [{
                xtype: "form",
                frame: true,
                ref: ".",
                fieldDefaults: {
                    msgTarget: 'side',
                    labelWidth: 70,
                },
                defaults: {
                    anchor: '100%',
                },
                items: [{
                    xtype: 'textfield',
                    id: 'playlistname',
                    fieldLabel: 'Playlist Name',
                    allowBlank: false,
                    value: playlist ? playlist.getName() : '',
                    readOnly: playlist ? true : false,
                }, {
                    xtype: 'textfield',
                    id: 'query',
                    fieldLabel: 'Query',
                    allowBlank: false,
                    value: playlist ? playlist.getQuery() : '',
                    emptyText: '(Group = "Chill" or Group = "Electronic") and Age < 7',
                }],
                buttons: [
                    {
                        text: 'Cancel',
                        id: 'cancel',
                        ref: '../..',
                        handler: function(b, e) {
                            window.close();
                        }
                    }, {
                        text: playlist ? 'Update' : 'Create',
                        id: "ok",
                        ref: "../..",
                        handler: function(b, e) {
                            var form = b.refOwner;
                            if (form.getForm().isValid()) {
                                var playlistname = playlist ? playlist.getName() : form.findById("playlistname").getValue();
                                var query = form.findById("query").getValue();
                                if (playlist) {
                                    qtunes.actionEditSmartPlaylist(playlistname, query);
                                } else if (qtunes.getNamedPlaylist(playlistname) != null) {
                                    Ext.Msg.alert("Error", "Playlist \""+playlistname+"\" already exists");
                                } else {
                                    qtunes.actionCreateSmartPlaylist(playlistname, query);
                                }
                                form.refOwner.close();
                            }
                        }
                    }],
            }]
        });
        window.show();
    },

};

}()));
